#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = 'Iván de Paz Centeno'

FACE_DETECTION_API = "http://visnode.sockhost.net:9909/detection-requests/faces/stream"
FACE_RECOGNITION_API = "http://visnode.sockhost.net:9909/recognition-requests/face/embedding/stream"
PORN_CLASSIFICATION_API = ""
